import com.sun.org.apache.xerces.internal.impl.dv.util.Base64;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.SignatureException;
import java.util.ArrayList;
import java.util.List;
import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import javax.net.ssl.HttpsURLConnection;
import java.util.HashMap;
import java.util.Iterator;


/**
 * This class allows you to connect with Bookitit webservices.
 * Only build an instance of the class, with your public and private key (obtained from Bookitit) as parameters,
 * and use the method you want.
 */
public class CRestClient {
    
        private String _sPublicKey;
        private String _sPrivateKey;
        private String _sRoot="app.bookitit.com/api/"; //url of bookitit web services. Example app.bookitit.com/api . No http protocol needed. No slashes at beginning and end.
        private String _sVersion = "11";
        
        public CRestClient(String p_sPublicKey, String p_sPrivateKey){
            this._sPublicKey=p_sPublicKey;
            this._sPrivateKey=p_sPrivateKey;
            this._sRoot += this._sVersion;
        }
        
        public String get_sRoot(){
            return this._sRoot;
        }
        
        public void set_sRoot(String p_sRoot)
        {
            this._sRoot = p_sRoot;
        }
        
        private String startConnection(String p_sUrl, String p_sMode, boolean p_bSecure, String p_sMethod, String p_sPostParams) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
                        
            String sSecure;
            
            if(p_bSecure==true){
                sSecure="https";
            }
            else{
                sSecure="http";
            }
            
            
            URI uri = new URI(sSecure, this._sRoot,"/"+p_sUrl,null);
            URL encodedUrl = uri.toURL();
            
            HttpURLConnection oConn;
            
            String sResult="";
            if(p_bSecure==true){
                oConn = (HttpsURLConnection) encodedUrl.openConnection();
                
            }
            else{
                oConn = (HttpURLConnection) encodedUrl.openConnection();
            }
            
            String sHashedPassword=calculateMD5HMAC(p_sUrl, _sPrivateKey);
            String sAuth=_sPublicKey+":"+sHashedPassword;
            sAuth = Base64.encode(sAuth.getBytes());
            oConn.setRequestProperty("Authorization", "Basic "+sAuth);
            oConn.setRequestProperty("Accept", "application/"+p_sMode);
            oConn.setRequestMethod(p_sMethod);
            oConn.setRequestProperty("charset", "utf-8");
            oConn.setDoOutput(true);


            if(p_sMethod.compareToIgnoreCase("POST")==0){
                oConn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded"); 
                oConn.setDoOutput(true);
                OutputStreamWriter wr = new OutputStreamWriter(oConn.getOutputStream());
                wr.write(p_sPostParams);
                wr.flush();
            }

            oConn.connect();
            BufferedReader reader = new BufferedReader(new InputStreamReader(oConn.getInputStream()));
            String sInputLine;
            while ((sInputLine = reader.readLine()) != null){ 
                sResult+=sInputLine;
            }
            reader.close();
            return sResult;
        }
        
        private static String calculateMD5HMAC(String data, String key) throws java.security.SignatureException, NoSuchAlgorithmException, InvalidKeyException, UnsupportedEncodingException{
            SecretKeySpec keySpec = new SecretKeySpec(key.getBytes(),"HmacMD5");

            Mac mac = Mac.getInstance("HmacMD5");
            mac.init(keySpec);
            byte[] bResult = mac.doFinal(data.getBytes());
            
            String result = "";
            for (int i=0; i < bResult.length; i++) {
            result +=  Integer.toString( ( bResult[i] & 0xff ) + 0x100, 16).substring( 1 );
            }
            
            return result;
        }
        
     /**
     * This function tests the connection.
     * The sent value is just echoed by the server.
     *
     * @param p_sEchoValue A value defined by the user.
     * @return
     * - on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <connection>
     *       - <status>true</status>
     *       - <echovalue>p_sEchoValue</echovalue>
     *     - </connection>
     *
     * - on success, if json was chosen:
     *   - {"connection":
     *     - {
     *       - "status":true,
     *       - "echovalue":p_sEchoValue
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - nothing (can't return data if the connection fails)
     *
     * - on failure, if json was chosen:
     *   - nothing (can't return data if the connection fails)
     */        
        public String testConnection(String p_sYourString, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl = "testconnection/"+p_sYourString;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
            return sResult;
        }
        
     /**
     * This function adds an agenda for your company.
     *
     * @param p_sName agenda name
     * @param p_sPhone agenda phone (optional)
     * @param p_sEmail agenda email (optional)
     * @param p_sPublic (optional) indicates if the agenda is "public" or "private". Default is "public"
     * @param p_sSynchroid (optional) id of the agenda in your software (MUST USE in case of an integration or synchronization between your software and Bookitit)
     * @param p_sServiceid (optional) id of a previously created service, which associate with this agenda (format bktXXXXXXX)
     *
     * @return
     * on success, if xml was chosen:
     * - <agenda>
     *   - <status>true</status>
     *   - <id>id of the created agenda (format bktXXXXXXXX)</id>
     * - </agenda>
     *
     * - on success, if json was chosen:
     *   - {"agenda":
     *     - {
     *       - "status":true,
     *       - "id":id of the created agenda (format bktXXXXXXXX)
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <agenda>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </agenda>
     *
     * - on failure, if json was chosen:
     *   - {"agenda":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */
        public String addAgenda(String p_sName, String p_sPhone, String p_sEmail, String p_sPublic, String p_sSynchroID, String p_sServiceID, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="addagenda/"+_sPublicKey;
            String postParams="p_sName="+p_sName+"&p_sPhone="+p_sPhone+"&p_sEmail="+p_sEmail+"&p_sPublic="+p_sPublic+"&p_sSynchroID="+p_sSynchroID+"&p_sServiceID="+p_sServiceID;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
        }
        
     /**
     * This function connects an agenda and a service for your company. Both must be already created, and its IDs supplied.
     * It can be the IDs of the agenda and the service in Bookitit or, if you are synchronizing/integrating services between your software and Bookitit, the ID of the agenda and the service in your software (we call it Synchro IDs).
     * If both IDs are provided, the Synchro IDs have priority.
     *
     * @param p_sAgendaID agenda ID (format bktXXXXXXX)
     * @param p_sServiceID service ID (format bktXXXXXXX)
     * @param p_sAgendaSynchroID agenda ID in your software (use in case of software integration)
     * @param p_sServiceSynchroID service ID in your software (use in case of software integration)
     * @param p_iDuration (optional) Duration of the service specific for this agenda, in minutes.
     * @param p_dPrice (optional) Price of the service specific for this agenda, in your currency
     *
     * @return
     * on success, if xml was chosen:
     * - <agenda_service>
     *   - <status>true</status>
     * - </agenda_service>
     *
     * - on success, if json was chosen:
     *   - {"agenda_service":
     *     - {
     *       - "status":true,
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <agenda_service>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </agenda_service>
     *
     * - on failure, if json was chosen:
     *   - {"agenda_service":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */
        public String addAgendaService(String p_sAgendaID,String p_sServiceID,String p_sAgendaSynchroID,String p_sServiceSynchroID,int p_iDuration, double p_dPrice, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="addagendaservice/"+_sPublicKey;
            String postParams="p_sAgendaID="+p_sAgendaID+"&p_sServiceID="+p_sServiceID+"&p_sAgendaSynchroID="+p_sAgendaSynchroID+"&p_sServiceSynchroID="+p_sServiceSynchroID+"&p_iDuration="+p_iDuration+"&p_dPrice="+p_dPrice;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
        }
        
     /**
     * This function is used to add a new company to Bookitit.
     * Only administrator users, who have an administrator api key, can create
     * companies with this function.
     *
     * @param p_sEmail email of the company
     * @param p_sPassword Encrypted password (md5) of the company.
     * @param p_sName Name of the company
     * @param p_sCellPhone Cellphone of the company (optional)
     * @param p_sPhone Phone of the company (optional)
     * @param p_sExpirationDate the expiration date of the account (format YYYY-MM-DD). Optional, default is one year from today.
     *
     * @return
     * - on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <company>
     *       - <status>true</status>
     *       - <id>id of the created company (format bkt XXXXXXXXX)</id>
     *       - <public>the public key of the created company</public>
     *       - <private>the private key of the created company</private>
     *     - </company>
     *
     * - on success, if json was chosen:
     *   - {"company":
     *     - {
     *       - "status":true,
     *       - "id":id of the created company (format bkt XXXXXXXXX),
     *       - "public":the public key of the created company,
     *       - "private":the private key of the created company
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <company>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </company>
     *
     * - on failure, if json was chosen:
     *   - {"company":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */
        public String addCompany(String p_sName,String p_sCellPhone, String p_sPhone, String p_sEmail, String p_sPassword, String p_sExpirationDate,int p_iDisabled, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="addcompany/"+_sPublicKey;
            String postParams="p_sName="+p_sName+"&p_sCellPhone="+p_sCellPhone+"&p_sPhone="+p_sPhone+"&p_sEmail="+p_sEmail+"&p_sPassword="+p_sPassword+"&p_sExpirationDate="+p_sExpirationDate+"&p_iDisabled="+p_iDisabled;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
        }
        
     /**
     * This function creates an event.
     *
     * @warning If you are creating an event for a social network, you might want to use the "addsocialevent" function instead.
     *
     * @param p_sAgendaID ID of the agenda (format bktXXXXXXXX)
     * @param p_sServiceID ID of the service (format bktXXXXXXXX)
     * @param p_dStartDate start date of the event (YYYY-MM-DD), if date doesn't exist, date will be 0000-00-00
     * @param p_dEndDate end date of the event (YYYY-MM-DD), if date doesn't exist, date will be 0000-00-00
     * @param p_iStartTime start time of the event (in minutes, since 00:00): 01:00 am = 60, 02:00 am = 120 ...
     * @param p_iEndTime end time of the event (in minutes, since 00:00): 01:00 am = 60, 02:00 am = 120 ...
     * @param p_sTitle (optional) title of the event, if it's set p_sServiceID the title will get automatically the name of the service. @deprecated the p_sServiceID is allways needed.
     * @param p_sDescription (optional) description of the event @deprecated, use p_sComments
     * @param p_sComments (optional) comments of the event
     * @param p_sEventSynchroID (optional) In case you are synchronizing your software with Bookitit, provide your event ID.
     * @param p_sAgendaSynchroID (optional) (mandatory if $p_sAgendaID is not set) In case you are synchronizing your software with Bookitit, provide your agenda ID.
     * @param p_sServiceSynchroID (optional) (mandatory if $p_sServiceID is not set) In case you are synchronizing your software with Bookitit, provide your service ID.
     * @param p_sUserID (optional) ID of the final user who makes the event (format bktXXXXXXXX) @deprecated, use p_sClientID
     * @param p_sUserPhone (optional) a phone number of the final user who makes the event. @deprecated
     * @param p_sClientID (optional) ID of the client who makes the event (format bktXXXXXXXX) 
     * @param p_sClientName (optional) name of the client, to create the new client if p_sUserID or p_sClientID is not set
     * @param p_sClientPhone (optional) phone of the client, to create the new client if p_sUserID or p_sClientID is not set
     * @param p_sClientEmail (optional) email of the client, to create the new client if p_sUserID or p_sClientID is not set
     * @param p_someCustomFields (optional) If your company account is using custom fields for the users, you can set it there.
     * 
     * $p_someCustomFields param example:
     *      array(                                                    
                "customevent1" => "data in text format",
                "customevent2"=> "data in text format", 
                "customevent3"=> "data in text format", 
                "customevent4"=> "data in text format", 
                "customevent5"=> "data in text format" 
            )
     *
     * @return
     * on success, if xml was chosen:
     * - <event>
     *   - <status>true</status>
     *   - <id>id of the created event (format bktXXXXXXXX)</id>
     * - </event>
     *
     * - on success, if json was chosen:
     *   - {"event":
     *     - {
     *       - "status":true,
     *       - "id":id of the created event (format bktXXXXXXXX)
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <event>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </event>
     *
     * - on failure, if json was chosen:
     *   - {"event":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */
        public String addEvent(String p_sAgendaID,String p_sServiceID,String p_dStartDate,String p_dEndDate,int p_iStartTime,int p_iEndTime,String p_sTitle,String p_sDescription,String p_sComments,String p_sEventSynchroID,String p_sAgendaSynchroID,String p_sServiceSynchroID,String p_sUserID,String p_sUserPhone,String p_sClientID,String p_sClientName,String p_sClientPhone,String p_sClientEmail, String p_sMode, boolean p_bSecure, HashMap<String, String> p_someCustomFields) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="addevent/"+_sPublicKey;
            String postParams="p_sAgendaID="+p_sAgendaID+"&p_sServiceID="+p_sServiceID+"&p_dStartDate="+p_dStartDate+"&p_dEndDate="+p_dEndDate+"&p_iStartTime="+p_iStartTime+"&p_iEndTime="+p_iEndTime+"&p_sTitle="+p_sTitle+"&p_sDescription="+p_sDescription+"&p_sComments="+p_sComments+"&p_sEventSynchroID="+p_sEventSynchroID+"&p_sAgendaSynchroID="+p_sAgendaSynchroID+"&p_sServiceSynchroID="+p_sServiceSynchroID+"&p_sUserID="+p_sUserID+"&p_sUserPhone="+p_sUserPhone+"&p_sClientID="+p_sClientID+"&p_sClientName="+p_sClientName+"&p_sClientPhone="+p_sClientPhone+"&p_sClientEmail="+p_sClientEmail;
            
            if (p_someCustomFields != null) {
                Iterator<String> keySetIterator = p_someCustomFields.keySet().iterator();

                while(keySetIterator.hasNext()){
                  String key = keySetIterator.next();                  
                  postParams += "&p_someCustomFields["+key+"]="+p_someCustomFields.get(key);
                }
            }
            
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
        }

    /**
     * This function creates an order for your company when a client intends to book a prepay service.
     * The order is created in an initial state (not paid).
     *
     * @return
     * on success, if xml was chosen:
     * - <order>
     *   - <status>true</status>
     *   - <id>id of the created order (format bktXXXXXXXX)</id>
     * - </order>
     *
     * - on success, if json was chosen:
     *   - {"order":
     *     - {
     *       - "status":true,
     *       - "id":id of the created order (format bktXXXXXXXX)
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <order>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </order>
     *
     * - on failure, if json was chosen:
     *   - {"order":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */
        public String addOrder(String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="addorder/"+_sPublicKey;
            String postParams="";
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
        }
        
     /**
     * This function adds a service for your company.
     *
     * @param p_sName name of the service
     * @param p_iTime duration of the service in minutes
     * @param p_dPrice price of the service (only numbers in format XX.XX)
     * @param p_sCurrency in which currency would be the service the price, accepted values are "EUR" for euro "GBP" for british pound and "USD" for United States dollar. Default is euro.
     * @param p_sPublic (optional) service marked as "public" or "private". Default is "public".
     * @param p_iPrepay (optional) "prepay" or "not_prepay". Default is "not_prepay".
     * @param p_sSynchroID (optional) id of the service in your software (MUST USE in case of an integration or synchronization between your software and Bookitit)
     *
     * @return
     * on success, if xml was chosen:
     * - <service>
     *   - <status>true</status>
     *   - <id>id of the created service (format bktXXXXXXXX)</id>
     * - </service>
     *
     * - on success, if json was chosen:
     *   - {"service":
     *     - {
     *       - "status":true,
     *       - "id":id of the created service (format bktXXXXXXXX)
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <service>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </service>
     *
     * - on failure, if json was chosen:
     *   - {"service":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */
        public String addService(String p_sName, int p_iTime, double p_dPrice, String p_sPublic, String p_sPrepay, String p_sCurrency, String p_sSynchroID, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="addservice/"+_sPublicKey;
            String postParams="p_sName="+p_sName+"&p_iTime="+p_iTime+"&p_dPrice="+p_dPrice+"&p_sPublic="+p_sPublic+"&p_sPrepay="+p_sPrepay+"&p_sCurrency="+p_sCurrency+"&p_sSynchroID="+p_sSynchroID;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
        }
                                        
     /**
     * This function creates a "social" event. The user must be a "social" user.
     *
     * A "social" event will always show for a user in different companies of the same social network.
     *
     * @warning This function will only create the event if the user is a "social" user.  If you are creating the event for a normal, single-site user, please use the "addevent" function instead.
     *
     * @param p_sPublicKey Your public key
     * @param p_sUserEmail User's email
     * @param p_sAgendaID id of the agenda (format bktXXXXXXXX)
     * @param p_sServiceID id of the service (format bktXXXXXXXX)
     * @param p_dStartDate start date of the event (YYYY-MM-DD)
     * @param p_dEndDate end date of the event (YYYY-MM-DD)
     * @param p_iStartTime start time of the event (in minutes, since 00:00)
     * @param p_iEndTime end time of the event (in minutes, since 00:00)
     * @param p_sTitle title of the event
     * @param p_sDescription description of the event
     *
     * @return
     * on success, if xml was chosen:
     * - <event>
     *   - <status>true</status>
     *   - <id>id of the created event (format bktXXXXXXXX)</id>
     * - </event>
     *
     * - on success, if json was chosen:
     *   - {"event":
     *     - {
     *       - "status":true,
     *       - "id":id of the created event (format bktXXXXXXXX)
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <event>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </event>
     *
     * - on failure, if json was chosen:
     *   - {"event":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */
        public String addSocialEvent(String p_sEmail,String p_sAgendaID,String p_sServiceID,String p_dStartDate,String p_dEndDate,int p_iStartTime,int p_iEndTime,String p_sComments,String p_sTitle,String p_sDescription, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="addsocialevent/"+_sPublicKey;
            String postParams="p_sEmail="+p_sEmail+"&p_sAgendaID="+p_sAgendaID+"&p_sServiceID="+p_sServiceID+"&p_dStartDate="+p_dStartDate+"&p_dEndDate="+p_dEndDate+"&p_iStartTime="+p_iStartTime+"&p_iEndTime="+p_iEndTime+"&p_sComments="+p_sComments+"&p_sTitle="+p_sTitle+"&p_sDescription="+p_sDescription;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
        }
        
     /**
     * This function adds a "social" user given his basic data.
     * A "social" user can validate his Bookitit account for the different companies of the same social network.
     *
     * Initially, the user will be inserted but INACTIVE.
     * An activation code will be sent to the given phone number.
     *
     * @warning If you are adding a user for a single site, please use the "adduser" function instead.
     * 
     * @param p_sName Name of the user
     * @param p_sPhone Phone number of the user
     * @param p_sEmail email of the user
     * @param p_sPassword Encrypted (md5) password for the user
     *
     * @return
     * - on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <user>
     *       - <status>true</status>
     *       - <id>id of the created user (format bktXXXXXXXXX)</id>
     *       - <validatephonekey>generated key for the user to validate his phone</validatephonekey>
     *     - </user>
     *
     * - on success, if json was chosen:
     *   - {"user":
     *     - {
     *       - "status":true,
     *       - "id":id of the created user (format bktXXXXXXXXX)
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *   - <user>
     *     - <status>false</status>
     *     - <id>error id</id>
     *     - <message>error message</message>
     *   - </user>
     *
     * - on failure, if json was chosen:
     *   - {"user":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     * @see adduser
     * @see setphoneandsendvalidate
     * @see confirmphone
     */
        public String addSocialUser(String p_sName, String p_sPhone, String p_sEmail, String p_sPassword, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="addsocialuser/"+_sPublicKey;
            String postParams="p_sName="+p_sName+"&p_sPhone="+p_sPhone+"&p_sEmail="+p_sEmail+"&p_sPassword="+p_sPassword;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
        }
        
    /**
     * This function adds a user given his basic data.
     * 
     * Initially, the user will be inserted but INACTIVE.
     * An activation code will be sent to the given phone number.
     *
     * @warning If you are adding a user for a social network, you might want to use the "addsocialuser" function instead.
     * 
     * @param p_sName Name of the user     
     * @param p_sEmail email of the user
     * @param p_sPassword Encrypted (md5) password for the user
     * @param p_sMode get response data in xml or json
     * @param p_bSecure connection security, true for https, false for http
     * @param p_sPhone (optional) Phone number of the user. If you don't want to send this parameter set is as "".
     * @param p_sDocument (optional) Document number of the user. If you don't want to send this parameter set is as "".
     * @param p_sAddress (optional) User address. If you don't want to send this parameter set is as "".
     * @param p_someCustomFields (optional) If your company account is using custom fields for the users, you can set it there. Remember that you have to maintain the index label.
            HashMap<String, String> someHash = new HashMap<String, String>();            
            someHash.put("customvalidate1", "yourvalue");
            someHash.put("customvalidate2", "yourvalue");
            someHash.put("custom1", "yourvalue");
            someHash.put("custom2", "yourvalue");
            someHash.put("custom3", "yourvalue");
            someHash.put("custom4", "yourvalue");
            someHash.put("custom5", "yourvalue");    
     *
     * @return
     * - on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <user>
     *       - <status>true</status>
     *       - <id>id of the created user (format bktXXXXXXXXX)</id>
     *     - </user>
     *
     * - on success, if json was chosen:
     *   - {"user":
     *     - {
     *       - "status":true,
     *       - "id":id of the created user (format bktXXXXXXXXX)
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <user>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </user>
     *
     *
     * - on failure, if json was chosen:
     *   - {"user":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     * @see addsocialuser
     * @see setphoneandsendvalidate
     * @see confirmphone
     */
      public String addUser(String p_sName, String p_sEmail, String p_sPassword, String p_sMode, 
              boolean p_bSecure, String p_sPhone, String p_sDocument, String p_sAddress, HashMap<String, String> p_someCustomFields) 
              throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
          
            String sUrl="adduser/"+_sPublicKey;
            String postParams="p_sName="+p_sName+"&p_sPhone="+p_sPhone+"&p_sEmail="+p_sEmail+"&p_sPassword="+p_sPassword+
                    "&p_sDocument="+p_sDocument+"&p_sAddress="+p_sAddress;    
            if (p_someCustomFields != null) {
                Iterator<String> keySetIterator = p_someCustomFields.keySet().iterator();

                while(keySetIterator.hasNext()){
                  String key = keySetIterator.next();                  
                  postParams += "&p_someCustomFields["+key+"]="+p_someCustomFields.get(key);
                }
            }
            
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
      }
        
        
     /**
     * This function changes the email of one of your users (if a user's ID is given) or the email of your company (if no ID is given).
     * Also marks the user or company as ACTIVE.
     * It should be used after the user or company enters the activation code received in a cellphone.
     *
     * @param p_sEmail email of the user
     * @param p_sUserID the id of the user (format bktXXXXXXXXX)
     *
     * @return
     * - on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <user>
     *       - <status>true</status>
     *     - </user>
     *
     * - on success, if json was chosen:
     *   - {"user":
     *     - {
     *       - "status":true
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <user>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </user>
     *
     * - on failure, if json was chosen:
     *   - {"user":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     * @see setphoneandsendvalidate
     */
        public String confirmEmail(String p_sUserID,String p_sEmail,String p_sRemember, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="confirmemail/"+_sPublicKey;
            String postParams="p_sUserID="+p_sUserID+"&p_sEmail="+p_sEmail+"&p_sRemember="+p_sRemember;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
        }
        
     /**
     * This function is used to set an event of your company to status CONFIRMED.
     * If synchronization is activated for your company, it will try to send the event to your local software. 
     *
     * @param p_sEventID the id of the event (format "bktXXXXXXXX")
     *
     * @return
     * - on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <event>
     *       - <status>true</status>
     *     - </event>
     *
     * - on success, if json was chosen:
     *   - {"event":
     *     - {
     *       - "status":true
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <event>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </event>
     *
     * - on failure, if json was chosen:
     *   - {"event":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     */
        public String confirmEvent(String p_sEventID, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="confirmevent/"+_sPublicKey;
            String postParams="p_sEventID="+p_sEventID;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
        }
        
     /**
     * This function changes the phone of one of your users (if a user's ID is given) or the phone of your company (if no ID is given).
     * Also marks the user or company as ACTIVE.
     * It should be used after the user or company enters the activation code received in a cellphone.
     *
     * @param p_sPhone Phone number of the user
     * @param p_sUserID the id of the user (optional, format bktXXXXXXXXX)
     *
     * @return
     * - on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <user>
     *       - <status>true</status>
     *     - </user>
     *
     * - on success, if json was chosen:
     *   - {"user":
     *     - {
     *       - "status":true
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <user>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </user>
     *
     * - on failure, if json was chosen:
     *   - {"user":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     * @see setphoneandsendvalidate
     */        
        public String confirmPhone(String p_sUserID, String p_sPhone, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="confirmphone/"+_sPublicKey;
            String postParams="p_sUserID="+p_sUserID+"&p_sPhone="+p_sPhone;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
        }
        
     /**
     * This function deletes an agenda of your company.
     * An agenda ID must be provided. It can be the ID of the agenda in Bookitit or, if you are synchronizing or integrating agendas between your software and Bookitit, the ID of the agenda in your software (we call it Synchro ID). If both are provided, the Synchro ID has preference.
     * 
     * @param type p_sAgendaID ID of the agenda (format bktXXXXXXX)
     * @param type p_sSynchroID ID of the agenda in your software
     * 
     * @return
     * on success, if xml was chosen:
     * - <agenda>
     *   - <status>true</status>
     * - </agenda>
     *
     * - on success, if json was chosen:
     *   - {"agenda":
     *     - {
     *       - "status":true,
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <agenda>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </agenda>
     *
     * - on failure, if json was chosen:
     *   - {"agenda":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     */
        public String deleteAgenda(String p_sAgendaID, String p_sSynchroID, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="deleteagenda/"+_sPublicKey;
            String postParams="p_sAgendaID="+p_sAgendaID+"&p_sSynchroID="+p_sSynchroID;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
        }
        
     /**
     * This function deletes an agenda-service connection of your company.
     * Agenda ID and service ID must be provided. They can be the IDs in Bookitit or, if you are synchronizing or integrating agendas between your software and Bookitit, the IDs of your software (we call it Synchro ID).
     * If both are given, Synchro IDs have preference.
     * 
     * @param p_sAgendaID agenda ID (format bktXXXXXXX)
     * @param p_sServiceID service ID (format bktXXXXXXX)
     * @param p_sAgendaSynchroID agenda ID in your software (use in case of software integration)
     * @param p_sServiceSynchroID service ID in your software (use in case of software integration)
     *  
     * @return
     * on success, if xml was chosen:
     * - <agenda_service>
     *   - <status>true</status>
     * - </agenda_service>
     *
     * - on success, if json was chosen:
     *   - {"agenda_service":
     *     - {
     *       - "status":true,
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <agenda_service>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </agenda_service>
     *
     * - on failure, if json was chosen:
     *   - {"agenda_service":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     * 
     * 
     */
        public String deleteAgendaService(String p_sAgendaID,String p_sServiceID,String p_sAgendaSynchroID,String p_sServiceSynchroID, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="deleteagendaservice/"+_sPublicKey;
            String postParams="p_sAgendaID="+p_sAgendaID+"&p_sServiceID="+p_sServiceID+"&p_sAgendaSynchroID="+p_sAgendaSynchroID+"&p_sServiceSynchroID="+p_sServiceSynchroID;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
        }
      
     /**
     * This function deletes an event of your company.
     *
     * @param p_iEventID id of the event to delete (format bktXXXXXXX)
     * @param p_sEventSynchroID id of the event in your software (use in case of software integration)
     * @param p_bSendNotification (optional) Indicates if an email will be sent to notify the client
     *                            "true": send email
     *                            "false": don't send email
     *                            default is "false"
     *
     * @return
     * on success, if xml was chosen:
     * - <event>
     *   - <status>true</status>
     * - </event>
     *
     * - on success, if json was chosen:
     *   - {"event":
     *     - {
     *       - "status":true,
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <event>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </event>
     *
     * - on failure, if json was chosen:
     *   - {"event":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */
        public String deleteEvent(String p_sEventID,String p_sEventSynchroID,String p_bSendNotification, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="deleteevent/"+_sPublicKey;
            String postParams="p_sEventID="+p_sEventID+"&p_sEventSynchroID="+p_sEventSynchroID+"&p_bSendNotification="+p_bSendNotification;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
        }
        
     /**
     * This function deletes a service of your company.
     * A service ID must be provided. It can be the ID of the service in Bookitit or, if you are synchronizing or integrating services between your software and Bookitit, the ID of the service in your software (we call it Synchro ID). If both are provided, the Synchro ID has preference.
     * 
     * @param type p_sServiceID ID of the service (format bktXXXXXXX)
     * @param type p_sSynchroID ID of the service in your software
     * 
     * @return
     * on success, if xml was chosen:
     * - <service>
     *   - <status>true</status>
     * - </service>
     *
     * - on success, if json was chosen:
     *   - {"service":
     *     - {
     *       - "status":true,
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <service>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </service>
     *
     * - on failure, if json was chosen:
     *   - {"service":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     */        
        public String deleteService(String p_sServiceID,String p_sSynchroID, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="deleteservice/"+_sPublicKey;
            String postParams="p_sServiceID="+p_sServiceID+"&p_sSynchroID="+p_sSynchroID;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
        }
        
     /**
     * This function gets the agendas of your company. Optionally, if a service ID is given, it will filter the agendas that offer that service.
     *
     * @param p_sServiceID optional, a service id (format "bktXXXXXXX")
     *
     * @return
     * -on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <agendas>
     *       - <status>true</status>
     *       - <i>one or more</i> <agenda>
     *         - <id>id of the agenda (format "bktXXXXXXX")</id>
     *         - <name>name of the agenda</name>
     *         - <phone>mobile phone of the agenda</phone>
     *         - <email>email of the agenda</email>
     *         - <public>indicates if the agenda is public (1) or private (0)</public>
     *         - <synchro_id>id of the agenda if it is shared with other software, NULL otherwise</synchro_id>
     *         - <photo>url of the photo</photo>
     *         - <description>description of the agenda</description>
     *       - </agenda>
     *     - </agendas>
     *
     * - on success, if json was chosen:
     *   - {"agendas":
     *     - {
     *       - "status":false,
     *       - <i>one or more</i> "agenda":
     *         - {
     *           - id:id of the agenda (format "bktXXXXXXX"),
     *           - name: name of the agenda,
     *           - phone: mobile phone of the agenda
     *           - email: email of the agenda
     *           - public:indicates if the agenda is public (1) or private (0)
     *           - synchro_id:id of the agenda if it is shared with other software, NULL otherwise
     *           - photo:url of the photo,
     *           - description:description of the agenda
     *         - }
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <agendas>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </agendas>
     *
     * - on failure, if json was chosen:
     *   - {"agendas":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */        
        public String getAgendas(String p_sServiceID, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl = "getagendas/"+_sPublicKey;
            if(p_sServiceID!=null && p_sServiceID.compareTo("")!=0){
                sUrl+="/"+p_sServiceID;
            }
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
            return sResult;
        }
        
     /**
     * This function gets the agenda-service assignments for your company.
     *
     * @return
     * -on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <agenda_services>
     *       - <status>true</status>
     *       - <i>one or more</i> <agenda_service>
     *         - <agendas_id>id of the agenda (format "bktXXXXXXX")</agendas_id>
     *         - <services_id>id of the service (format "bktXXXXXXX")</services_id>
     *         - <agendas_synchro_id>id of the agenda in your software (in case of software integration)</agendas_synchro_id>
     *         - <services_synchro_id>id of the service in your software (in case of software integration)</services_synchro_id>
     *         - <duration>duration of the service (specific for this agenda)</duration>
     *         - <price>price of the service (specific for this agenda)</price>
     *       - </agenda_service>
     *     - </agenda_services>
     *
     * - on success, if json was chosen:
     *   - {"agenda_services":
     *     - {
     *       - "status":true,
     *       - <i>one or more</i> "agenda_service":
     *         - {
     *           - agendas_id: id of the agenda (format "bktXXXXXXX"),
     *           - services_id: id of the service (format "bktXXXXXXX"),
     *           - agendas_synchro_id: id of the agenda in your software (in case of software integration),
     *           - services_synchro_id: id of the service in your software (in case of software integration),
     *           - duration: duration of the service (specific for this agenda),
     *           - price: price of the service (specific for this agenda),
     *         - }
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <agenda_services>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </agenda_services>
     *
     * - on failure, if json was chosen:
     *   - {"agenda_services":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */        
        public String getAgendasServices(String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl = "getagendasservices/"+_sPublicKey;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
            return sResult;
        }
        
     /**
     * This function gets the cities of a given region, in a given language, from the Bookitit city database
     *
     * @param p_sLanguageID ID of the chosen language (format bktXXXXXX)
     * @param p_sRegionID ID of the chosen region (format bktXXXXXX)
     *
     * @return
     * -on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <cities>
     *       - <status>true</status>
     *       - <i>one or more</i> <citie>
     *         - <id>id of the city (format "bktXXXXXXX")</id>
     *         - <name>name of the city in the chosen language</name>
     *         - <longitude>longitude of the city's center</longitude>
     *         - <latitude>latitude of the city's center</latitude>
     *       - </citie>
     *     - </cities>
     *
     * - on success, if json was chosen:
     *   - {"cities":
     *     - {
     *       - "status":false,
     *       - <i>one or more</i> "citie":
     *         - {
     *           - id:id of the city (format "bktXXXXXXX"),
     *           - name: name of the city in the chosen language,
     *           - longitude: longitude of the city's center,
     *           - latitude: latitude of the city's center,
     *         - }
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <cities>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </cities>
     *
     * - on failure, if json was chosen:
     *   - {"cities":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */          
        public String getCities(String p_sLanguageID, String p_sRegionID, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl = "getcities/"+_sPublicKey+"/"+p_sLanguageID+"/"+p_sRegionID;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
            return sResult;
        }
        
    /**
     * This function gets the data for all clients of your company.(max limit 30 days)
     *
     * @param p_sPublicKey Your public key
     * @param p_sType (optional) The type of public key that you are using. Options are "phone" and "standard". Default is "standard".
     * @param p_sStartDate the start day to get the clients (when the client was registered)
     * @param p_sEndDate the end day to get the clients (when the client was registered)
     *
     * @return
     * - on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <clients>
     *       - <status>true</status>
     *       - one or more <client>
     *         - <id>id of the client (format bktXXXXXXXXX)</id>
     *         - <user_id>id of the user (format bktXXXXXXXXX). If this parameter is void the client will be without web access, otherwise it will be a client with web access</user_id>
     *         - <name>name of the user</name>
     *         - <email>email of the user</email>
     *         - <phone>phone of the user</phone>
     *         - <document>document of the user</document>
     *         - <cellphone>cellphone of the user</cellphone>
     *         - <customvalidate1>user custom validate field 1</customvalidate1>
     *         - <customvalidateX>user custom validate field X</customvalidateX>
     *         - <custom1>user custom field 1</custom1>
     *         - <custom2>user custom field 2</custom2>
     *         - <customX>user custom field X</customX>
     *       - </client>
     *     - </clients>
     *
     * - on success, if json was chosen:
     *   - {"clients":
     *     - one or more {"client":
     *       - {
     *         - "status":true,
     *         - "id":id of the user (format bktXXXXXXXXX),
     *         - "user_id":id of the user (format bktXXXXXXXXX). If this parameter is void the client will be without web access, otherwise it will be a client with web access,
     *         - "name":name of the user,
     *         - "email":email of the user,
     *         - "phone":phone of the user,
     *         - "document":document of the user,
     *         - "cellphone":cellphone of the user,
     *         - "customvalidate1":user custom validate field 1,
     *         - "customvalidateX":user custom validate field X,
     *         - "custom1":user custom field 1,
     *         - "custom2":user custom field 2,
     *         - "customX":user custom field X
     *       - }
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <clients>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </clients>
     *
     * - on failure, if json was chosen:
     *   - {"clients":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     */
        public String getClients(String p_sMode, boolean p_bSecure, String p_sDateFrom, String p_sDateTo) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl = "getclients/"+_sPublicKey+"/"+p_sDateFrom+"/"+p_sDateTo;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
            return sResult;
        }
        
    /**
    * This function gets one client from its client id. The format is bktXXXXX
    * 
    * @param p_sIdClient the id of the client in bktXXXXXXXXX format
    * @param p_sMode xml or json
    * @param p_bSecure true for https, false for http
    *
    * @return
    * - on success, if xml was chosen:
    *   - <?xml version='1.0' encoding='utf-8'?>
        - <client>
    *          - <email>The email of the client</email>
    *          - <address>The address of the client</address>
    *          - <cellphone>The cellphone of the client</cellphone>
    *          - <phone>The phone of the client</phone>
    *          - <document>The document of the client</document>
    *          - <name>The name of the client</name>
    *          - <custom1>The first custom field</custom1>
    *          - <custom2>The second custom field</custom2>
    *          - <customN>The N custom field</customN>
    *          - <status>if success "true" otherwise "false"</status>
    *     - </client>
    *
    * - on success, if json was chosen:
    *    {"client":
    *      {"email":"The email of the client",
    *       "address":"The address of the client",
    *       "cellphone":"The cellphone of the client",
    *       "phone":"The phone of the client",
    *       "document":"The document of the client",
    *       "name":"Raulope",
    *       "custom1":"The first custom field",
    *       "custom2":"The second custom field",
    *       "customN":"The N custom field",
    *       "status":"true"}}
    *
    * - on failure, if xml was chosen:
    *   - <?xml version='1.0' encoding='utf-8'?>
    *     - <client>
    *       - <status>false</status>
    *       - <id>error id</id>
    *       - <message>error message</message>
    *     - </client>
    *
    * - on failure, if json was chosen:
    *   - {"client":
    *     - {
    *       - "status":false,
    *       - "id":error id,
    *       - "message":error message
    *     - }
    *   - }
    */
    public String getClient(String p_sIdClient, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
      String sUrl = "getclient/"+this._sPublicKey+"/"+p_sIdClient;        
      String sMethod = "GET";
      String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
      return sResult;
    }   
    
      /**
     * This function gets a client of one event. If you have an event and you want to know who went to the appointment, you must use this function.     
     *
     * @param p_sEventID the identifier of one event (format bktXXXXX)     
     * @param p_sMode xml or json
     * @param p_bSecure true for https, false for http
     *
     *
     * @return
     * -on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <clients>     
     *         - <status>true</status>     
     *         - <i>one or more</i> "client":
     *         - <client>
     *          - <name>the name of the client</name>
     *          - <email>the email of the client</email>
     *          - <address>the address of the client</address> 
     *          - <document>the document of the client</document>
     *          - <cellphone>the cellphone of the client</cellphone>
     *          - <phone>the phone of the client</phone>
     *          - <custom1>the first custom field of the client</custom1>
     *          - <customn>the N custom field of the client</customn>
     *          - <id>The id of the client</id>
     *          - <event_id>The id of the event</event_id>
     *          - <agendas_id>The id of the agenda where the event was created</agendas_id>
     *          - <start_date>The start date of the event</start_date>
     *          - <end_date>The end date of the event</end_date>
     *          - <start_time>660</start_time>
     *          - <end_time>720</end_time>
     *          - <title>The title of the event</title>
     *          - <description>The description of the event</description>
     *          - <coupons_code>The code of the coupon for the event</coupons_code>
     *          - <coupon_id>The id of the coupon related to the event</coupon_id>
     *          - <comments>The comments of the event</comments>
     *          - <synchro_id>the id of the event of your database</synchro_id>
     *          - <people>The people that went to the event</people>
     *          - <showup>The client went to the appointment, yes or not</showup>
     *          - <locator>The locator of the event</locator>
     *          - <price_total>The total price of the event</price_total>
     *          - <price_paid>The event price</price_paid>
     *          - <agenda_name>The name of the agenda selected for the event</agenda_name>
     *          - <customevent1>The first custom field of the event</customevent1>
     *          - <customeventN>The N custom field of the event</customeventN>
     *         - </client>
     *     - </clients>
     *
     * 
     * 
     * - on success, if json was chosen:
     *      -{"clients":
     *          -{"0":
     *              -{
     *                 "name":"the name of the client",
     *                 "email":"the email of the client",
     *                 "address":"the address of the client",
     *                 "document":"the document of the client",     
     *                 "cellphone":"the cellphone of the client",
     *                 "phone":"the phone of the client",     
     *                 "custom1":"the first custom field of the client",
     *                 "customN": "the N custom field of the client",
     *                 "id":"The id of the client",
     *                 "event_id":"The id of the event",
     *                 "agendas_id":"The id of the agenda where the event was created",
     *                 "services_id":"The id of the selected service for the event",
     *                 "start_date":"The start date of the event",
     *                 "end_date":"The end date of the event",
     *                 "start_time":"The start time of the event",
     *                 "end_time":"The end time of the event",
     *                 "title":"The title of the event",
     *                 "description":"The description of the event",
     *                 "coupons_code": "The code of the coupon for the event",
     *                 "coupon_id":"The id of the coupon",
     *                 "comments":"The comments inside the event",
     *                 "synchro_id":"the id of the event of your database",
     *                 "people":"the number of people that went to the event",
     *                 "showup":"The client went to the appointment, yes or not",
     *                 "locator":"The locator of the event",
     *                 "price_total":"The total price of the event",
     *                 "price_paid":"The event price",
     *                 "agenda_name":"The name of the agenda selected for the event",
     *                 "customevent1":"The first custom field of the event",
     *                 "customeventN":"The N custom field of the event",
     *              },
     *          -"status":"true"}}
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <clients>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </clients>
     *
     * - on failure, if json was chosen:
     *   - {"clients":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */
    public String getClientsByEvent(String p_sIdEvent, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
        String sUrl = "getclientsbyeventid/"+this._sPublicKey+"/"+p_sIdEvent;        
        String sMethod = "GET";
        String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
        return sResult;
    }
    
        
     /**
     * This function gets the countries of the Bookitit city database, in a given language
     *
     * @param p_sLanguageID ID of the language (format bktXXXXXX)
     *
     * @return
     * -on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <countries>
     *       - <status>true</status>
     *       - <i>one or more</i> <countrie>
     *         - <id>id of the country (format "bktXXXXXXX")</id>
     *         - <name>name of the country in the choosed language</name>
     *         - <longitude>longitude of the country's center</longitude>
     *         - <latitude>latitude of the country's center</latitude>
     *         - <phone_code>phone code of the country</phone_code>
     *       - </countrie>
     *     - </countries>
     *
     * - on success, if json was chosen:
     *   - {"countries":
     *     - {
     *       - "status":false,
     *       - <i>one or more</i> "countrie":
     *         - {
     *           - id:id of the country (format "bktXXXXXXX"),
     *           - name: name of the country,
     *           - longitude: longitude of the country's center,
     *           - latitude: latitude of the country's center,
     *           - phone_code: phone code of the country,
     *         - }
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <countries>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </countries>
     *
     * - on failure, if json was chosen:
     *   - {"countries":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */        
        public String getCountries(String p_sLanguageID, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl = "getcountries/"+_sPublicKey+"/"+p_sLanguageID;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
            return sResult;
        }
        
    /**
     * This function gets a list of events between two dates.
     * If no agenda ID is given, it will get the events of all the agendas. If an agenda ID is supplied, it will only get the events for that agenda.
     * For the second option it will be ordered by the id of the agenda.
     * If you want to get the events of a user you must use method "getEventsOfUser" instead.
     *
     * @param p_sPublicKey Your public key
     * @param p_sStartDate the start day to get the events
     * @param p_sEndDate the end day to get the events
     * @param p_sAgendaID (optional) ID of the agenda. If no ID is given, you will get all the events of your company.
     *
     * @return
     * -on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <events>
     *       - <status>true</status>
     *       - <i>one or more</i> 
     *        -<event>     
      *         - <id>id of the event (format "bktXXXXXXX")</id>
     *         - <title>title of the event, if exists. empty otherwise</title>
     *         - <description>description of the event, if exists. empty otherwise</description>
     *         - <comments>comments for the event, if exists. empty otherwise</comments>
     *         - <synchro_id>id of the event if it is shared with other software, empty otherwise</synchro_id>
     *         - <start_date>the start date of the event</startdate>
     *         - <end_date>the end time of the event</enddate>    
     *         - <start_time>the starttime of the event (format HH:MM)</starttime>
     *         - <end_time>the endtime of the event (format HH:MM)</endtime>     
     *         - <coupon>the coupon code of the offer, if exists. empty otherwise</coupon_code>     
     *         - <agenda_id>id of the agenda (format "bktXXXXX")</agenda_id>
     *         - <user_id>id of the client (format "bktXXXXX"), if exists. empty otherwise</user_id>
     *         - <user_name>name of the client, if exists. empty otherwise</user_name>
     *         - <user_cellphone>cellphone of the client, if exists. empty otherwise</user_cellphone>
     *         - <user_email>email of the client, if exists. empty otherwise</user_email>     
     *         - <service_id>id service of the event (format "bktXXXXX"), if exists. empty otherwise</services_id>     
     *         - <locator>locator of the event, if exists. empty otherwise</locator>
     *         - <updated>last update of the event</updated>
     *         - <customvalidate1>user custom validate field 1, if exists. empty otherwise</customvalidate1>
     *         - <customvalidateX>user custom validate field X, if exists. empty otherwise</customvalidateX>
     *         - <custom1>user custom field 1, if exists. empty otherwise</custom1>
     *         - <custom2>user custom field 2, if exists. empty otherwise</custom2>
     *         - <customX>user custom field X, if exists. empty otherwise</customX>
     *         - <customevent1>event custom field 1, if exists. empty otherwise</customevent1>
     *         - <customevent2>event custom field 2, if exists. empty otherwise</customevent2>
     *         - <customeventX>event custom field X, if exists. empty otherwise</customeventX>
     *       - </event>
     *     - </events>
     *
     * - on success, if json was chosen:
     *   - {"events":
     *     - {
     *       - "status":false,
     *       - <i>one or more</i> "events":
     *         - {
    *           - id: id of the event (format "bktXXXXXXX")
     *           - title: title of the event, if exists. empty otherwise
     *           - description: description of the event, if exists. empty otherwise
     *           - comments: comments for the event, if exists. empty otherwise
     *           - synchro_id: id of the event if it is shared with other software, empty otherwise
     *           - agenda_synchro_id: id of the agenda synchro if it is shared with other software, empty otherwise
     *           - start_date: the start date of the event
     *           - end_date: the end time of the event
     *           - start_time: the starttime of the event (format HH:MM)
     *           - end_time: the endtime of the event (format HH:MM)
     *           - coupon: the coupon code of the offer, if exists. empty otherwise
     *           - agenda_id: id of the agenda (format "bktXXXXX")
     *           - user_id: id of the client (format "bktXXXXX"), if exists. empty otherwise
     *           - user_name: name of the client, if exists. empty otherwise
     *           - user_cellphone: cellphone of the client, if exists. empty otherwise
     *           - user_email: email of the client, if exists. empty otherwise
     *           - service_id: id service of the event (format "bktXXXXX"), if exists. empty otherwise
     *           - locator: locator of the event, if exists. empty otherwise
     *           - updated: last update of the event
     *           - "customvalidate1": user custom validate field 1, if exists. empty otherwise
     *           - "customvalidateX": user custom validate field X, if exists. empty otherwise
     *           - "custom1": user custom field 1, if exists. empty otherwise
     *           - "custom2": user custom field 2, if exists. empty otherwise
     *           - "customX": user custom field X, if exists. empty otherwise
     *           - "customevent1": event custom field 1, if exists. empty otherwise
     *           - "customevent2": event custom field 2, if exists. empty otherwise
     *           - "customeventX": event custom field X, if exists. empty otherwise
     *         - }
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <agendas>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </agendas>
     *
     * - on failure, if json was chosen:
     *   - {"agendas":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */        
        public String getEvents(String p_sDateFrom, String p_sDateTo, String p_sAgendaID, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl = "getevents/"+_sPublicKey+"/"+p_sDateFrom+"/"+p_sDateTo;
            if(p_sAgendaID!=null && p_sAgendaID.compareTo("")!=0){
                sUrl+="/"+p_sAgendaID;
            }
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
            return sResult;
        }
        
     /**
     * This function gets a list of events of a user.
     *
     * @param p_iIdUser the id of the user with format bktXXXXX
     * @param p_sStartDate the start day to get the events
     * @param p_sEndDate the end day to get the events
     * @param p_sAgendaID (optional) ID of the agenda. If no ID is given, you will get all the events of your company.
     *
     * @return
     * -on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <events>
     *       - <status>true</status>
     *       - <i>one or more</i> <events>
     *          - <id>id of the event (format "bktXXXXXXX")</id>
     *         - <title>title of the event</name>
     *         - <description>description of the event</description>
     *         - <comments>comments of the event</comments>
     *         - <synchro_id>id of the event if it is shared with other software, NULL otherwise</synchro_id>
     *         - <startdate>start date of the event (format YYYY-MM-DD)</startdate>
     *         - <enddate>end date of the event</enddate>
     *         - <starttime>start date of the event (format HH:MM)</starttime>
     *         - <endtime>end date of the event</endtime>
     *         - <coupon_code>the coupon code</coupon_code>
     *         - <agenda_id>id of the agenda (format "bktXXXXX")</agenda_id>
     *         - <user_id>id of the user/client for the event(format "bktXXXXX")</user_id>
     *         - <user_name>name of the user/client</user_name>
     *         - <user_cellphone>phone of the user/client</user_cellphone>
     *         - <user_email>email of the user/client</user_email>
     *         - <service_id>id of the service</client</services_id>
     *         - <user_prof_id>id of the company which event belongs</user_prof_id>
     *       - </event>
     *     - </events>
     *
     * - on success, if json was chosen:
     *   - {"events":
     *     - {
     *       - "status":false,
     *       - <i>one or more</i> "events":
     *         - {
     *           - id:id of the event (format "bktXXXXXXX"),
     *           - title: title of the event,
     *           - description: description of the event, if exists. NULL otherwise
     *           - comments: comments for the event, if exists. NULL otherwise
     *           - synchro_id:id of the event if it is shared with other software, NULL otherwise
     *           - startdate:the start date of the event
     *           - enddate:the end time of the event
     *           - starttime:the starttime of the event
     *           - endtime:the endtime of the event
     *           - coupon_code:the coupon code of the offer, if exists. NULL otherwise.
     *           - agenda_id:if of the agenda(format "bktXXXXX")
     *           - user_id:id of the user/client(format "bktXXXXX")
     *           - user_name:name of the user/client
     *           - user_cellphone:cellphone of the user/client
     *           - user_email:email of the user/client
     *           - service_id: service of the event
     *           - user_prof_id: id of the company which event belongs {format bktXXXXX}
     *         - }
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <agendas>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </agendas>
     *
     * - on failure, if json was chosen:
     *   - {"agendas":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */         
        public String getEventsOfUser(String p_iIdUser, String p_sDateFrom, String p_sDateTo, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl = "geteventsofuser/"+_sPublicKey+"/"+p_iIdUser+"/"+p_sDateFrom+"/"+p_sDateTo;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
            return sResult;
        }
        
     /**
     * This function gets the free minutes for a service of an agenda
     *
     * @param p_sServiceID service id (format bktXXXXXXX)
     * @param p_sAgendaID agenda id (format bktXXXXXXX)
     * @param p_sDate the date to show the slots of (format YYYY-MM-DD)
     *
     * @return
     * on success, if xml was chosen:
     * - <slots>
     *   - <status>true</status>
     *   - <hours>
     *     - <i>one or more</i><hour>a time where a reservation could be made (format HH:MM)</hour>
     *   - </hours> 
     * - </slots>
     *
     * - on success, if json was chosen:
     *   - {"slots":
     *     - {
     *       - "status":true,
     *       - "hours":
     *         - {
     *           - <i>one or more</i>"hour": a time where a reservation could be made (format HH:MM)
     *         - }
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <slots>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </slots>
     *
     * - on failure, if json was chosen:
     *   - {"slots":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */        
        public String getFreeSlots(String p_iIdService, String p_iIdAgenda, String p_sDate, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl = "getfreeslots/"+_sPublicKey+"/"+p_iIdService+"/"+p_iIdAgenda+"/"+p_sDate;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
            return sResult;
        }
        
     /**
     * This function gets the languages of the Bookitit city database
     *
     * @return
     * -on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <languages>
     *       - <status>true</status>
     *       - <i>one or more</i> <language>
     *         - <id>id of the language (format "bktXXXXXXX")</id>
     *         - <code>international code of the language</code>
     *       - </language>
     *     - </languages>
     *
     * - on success, if json was chosen:
     *   - {"languages":
     *     - {
     *       - "status":false,
     *       - <i>one or more</i> "language":
     *         - {
     *           - id:id of the language (format "bktXXXXXXX"),
     *           - code: international code of the language,
     *         - }
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <languages>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </languages>
     *
     * - on failure, if json was chosen:
     *   - {"languages":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */         
        public String getLanguages(String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl = "getlanguages/"+_sPublicKey;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
            return sResult;
        }
        
     /**
     * This function gets the orders of your company
     *
     * @return
     * -on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <orders>
     *       - <status>true</status>
     *       - <i>one or more</i> <order>
     *         - <id>id of the order (format "bktXXXXXXX")</id>
     *         - <created>creation date of the order</created>
     *         - <updated>modification date of the order</updated>
     *         - <state>state of the event</state>
     *         - <events_id>id of the event associated to the order (format "bktXXXXXXX")</events_id>
     *         - <clients_id>id of the client associated to the order (format "bktXXXXXXX")</clients_id>
     *         - <transaction_id>id of the economic transaction in the payment platform</transaction_id>
     *       - </order>
     *     - </orders>
     *
     * - on success, if json was chosen:
     *   - {"orders":
     *     - {
     *       - "status":false,
     *       - <i>one or more</i> "order":
     *         - {
     *           - id: id of the order (format "bktXXXXXXX"),
     *           - created: creation date of the order,
     *           - updated: modification date of the order,
     *           - state: state of the event,
     *           - events_id: id of the event associated to the order (format "bktXXXXXXX"),
     *           - clients_id: id of the client associated to the order (format "bktXXXXXXX"),
     *           - transaction_id: id of the economic transaction in the payment platform,
     *         - }
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <orders>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </languages>
     *
     * - on failure, if json was chosen:
     *   - {"languages":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */         
        public String getOrders(String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl = "getorders/"+_sPublicKey;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
            return sResult;
        }
        
     /**
     * This function gets the regions of a given country, in a given language, from the Bookitit city database
     *
     * @param p_sLanguageID ID of the chosen language (format bktXXXXXX)
     * @param p_sCountryID ID of the chosen country (format bktXXXXXX)
     *
     * @return
     * -on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <regions>
     *       - <status>true</status>
     *       - <i>one or more</i> <region>
     *         - <id>id of the region (format "bktXXXXXXX")</id>
     *         - <name>name of the region in the chosen language</name>
     *         - <longitude>longitude of the region's center</longitude>
     *         - <latitude>latitude of the region's center</latitude>
     *         - <gmt>time from GMT</gmt>
     *       - </region>
     *     - </regions>
     *
     * - on success, if json was chosen:
     *   - {"regions":
     *     - {
     *       - "status":false,
     *       - <i>one or more</i> "region":
     *         - {
     *           - id:id of the region (format "bktXXXXXXX"),
     *           - name: name of the region in the chosen language,
     *           - longitude: longitude of the region's center,
     *           - latitude: latitude of the region's center,
     *           - gmt: time from GMT,
     *         - }
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <regions>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </regions>
     *
     * - on failure, if json was chosen:
     *   - {"regions":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */        
        public String getRegions(String p_sLanguageID, String p_sCountryID, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl = "getregions/"+_sPublicKey+"/"+p_sLanguageID+"/"+p_sCountryID;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
            return sResult;
        }
        
    /**
     * This function gets the services of your company
     *
     * @param p_sMode get response data in xml or json
     * @param p_bSecure connection security, true for https, false for http
     *
     * @return
     * - on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <services>
     *       - <status>true</status>
     *       - <i>one or more</i> <service>
     *         - <id>id of the service (format bktXXXXXXXX)</id>
     *         - <name>name of the service</name>
     *         - <duration>duration of the service in minutes</duration>
     *         - <prepay>flag to indicate if it is a prepay service:
     *           - 0: not prepay service
     *           - 1: prepay service
     *         - </prepay>
     *         - <price>price of the service</price>
     *       - </service>
     *     - </services>
     *
     * - on success, if json was chosen:
     *   - {"services":
     *     - {
     *       - "status":true,
     *       - <i>one or more</i> "service":
     *         - {
     *           - "id":id of the service (format bktXXXXXXXX),
     *           - "name":name of the service,
     *           - "duration":duration of the service in minutes,
     *           - "prepay": 0 (is not prepay service) or 1 (is prepay service),
     *           - "price":price of the service
     *         - }
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <services>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </services>
     *
     * - on failure, if json was chosen:
     *   - {"services":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */        
        public String getServices(String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl = "getservices/"+_sPublicKey;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
            return sResult;
        }
        
    /**
     * Only administrator users, who have an administrator api key, can invoke this method.
     * This function gets a list of events of a user in a social network between two dates. It will get the events of the user for all
     * the companies of the social network.
     *
     * @param p_sUserID the id of the user with format bktXXXXX
     * @param p_sStartDate the start day to get the events
     * @param p_sEndDate the end day to get the events
     * @param p_sAgendaID (optional) ID of the agenda. If no ID is given, you will get all the events of your company.
     *
     * @return
     * -on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <events>
     *       - <status>true</status>
     *       - <i>one or more</i> <events>
     *          - <id>id of the event (format "bktXXXXXXX")</id>
     *         - <title>title of the event</name>
     *         - <description>description of the event</description>
     *         - <comments>comments of the event</comments>
     *         - <synchro_id>id of the event if it is shared with other software, NULL otherwise</synchro_id>
     *         - <startdate>start date of the event (format YYYY-MM-DD)</startdate>
     *         - <enddate>end date of the event</enddate>
     *         - <starttime>start date of the event (format HH:MM)</starttime>
     *         - <endtime>end date of the event</endtime>
     *         - <coupon_code>the coupon code</coupon_code>
     *         - <agenda_id>id of the agenda (format "bktXXXXX")</agenda_id>
     *         - <user_id>id of the user/client for the event(format "bktXXXXX")</user_id>
     *         - <user_name>name of the user/client</user_name>
     *         - <user_cellphone>phone of the user/client</user_cellphone>
     *         - <user_email>email of the user/client</user_email>
     *         - <service_id>id of the service</client</services_id>
     *         - <user_prof_id>id of the company which event belongs</user_prof_id>
     *       - </event>
     *     - </events>
     *
     * - on success, if json was chosen:
     *   - {"events":
     *     - {
     *       - "status":false,
     *       - <i>one or more</i> "events":
     *         - {
     *           - id:id of the event (format "bktXXXXXXX"),
     *           - title: title of the event,
     *           - description: description of the event, if exists. NULL otherwise
     *           - comments: comments for the event, if exists. NULL otherwise
     *           - synchro_id:id of the event if it is shared with other software, NULL otherwise
     *           - startdate:the start date of the event
     *           - enddate:the end time of the event
     *           - starttime:the starttime of the event
     *           - endtime:the endtime of the event
     *           - coupon_code:the coupon code of the offer, if exists. NULL otherwise.
     *           - agenda_id:if of the agenda(format "bktXXXXX")
     *           - user_id:id of the user/client(format "bktXXXXX")
     *           - user_name:name of the user/client
     *           - user_cellphone:cellphone of the user/client
     *           - user_email:email of the user/client
     *           - service_id: service of the event
     *           - user_prof_id:id of the company which event belongs{format "bktXXXXX"}
     *         - }
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <agendas>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </agendas>
     *
     * - on failure, if json was chosen:
     *   - {"agendas":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */         
        public String getSocialEventsOfUser(String p_sUserID, String p_sDateFrom, String p_sDateTo, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl = "getsocialeventsofuser/"+_sPublicKey+"/"+p_sUserID+"/"+p_sDateFrom+"/"+p_sDateTo;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
            return sResult;
        }
        
     /**
     * This function gets the data of a user of your company.
     *
     * @param p_sPublicKey Your public key
     * @param p_sUserID Identifier of the user to retrieve data of. The format is bktXXXXXXXXX.
     *
     * @return
     * - on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <user>
     *       - <status>true</status>
     *       - <id>id of the user (format bktXXXXXXXXX)</id>
     *       - <name>name of the user</name>
     *       - <email>email of the user</email>
     *       - <phone>phone of the user</phone>
     *   - </user>
     *
     * - on success, if json was chosen:
     *   - {"user":
     *     - {
     *       - "status":true,
     *       - "id":id of the user (format bktXXXXXXXXX),
     *       - "name":name of the user,
     *       - "email":email of the user,
     *       - "phone":phone of the user
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <user>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </user>
     *
     * - on failure, if json was chosen:
     *   - {"user":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     */        
        public String getUser(String p_sIdUser, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl = "getuser/"+_sPublicKey+"/"+p_sIdUser;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
            return sResult;
        }
        
    /**
     * This function translates a number in minutes elapsed from 00:00 to hour format
     * @param p_iHourInMinutes the hour in minutes elapsed from 00:00
     * @return sTime string with the time (format HH:MM)
     */        
        public String renderHour(int p_iHourInMinutes) {
            String sTime;

            int iHours = p_iHourInMinutes/60;
            String sHours="";
            if(p_iHourInMinutes%60!=0){
                    iHours = (p_iHourInMinutes - (p_iHourInMinutes%60))/60;
                    if (iHours < 10) {
                            sHours = "0"+iHours;
                    }
            }
            else {
                    if (iHours < 10) {
                            sHours = "0"+iHours;
                    }
            }

            int iMinute = p_iHourInMinutes%60;
            if(iMinute==0){
                    sTime = sHours+":00";
            }else{
                    if(iMinute<10)
                    {
                            sTime = iHours+":0"+iMinute;
                    }
                    else
                    {
                            sTime = iHours+":"+iMinute;
                    }

            }

            return sTime;
        }
        
     /**
     * This function translates a time in format "HH:MM" into the number of minutes elapsed from 00:00
     * @param p_sTime the time (in format HH:MM)
     * @return iMinutes number of minutes elapsed from 00:00 to p_sTime
     */    
    public int renderMinutes(String p_sTime) {
        String[] someHoursMinutes = p_sTime.split(":");
        int iMinutes=(Integer.valueOf(someHoursMinutes[0]))*60;
        iMinutes = iMinutes + (Integer.valueOf(someHoursMinutes[1]));
        return iMinutes;
    }
    
     /**
     * This function makes an agenda schedule to be the same as the previously set company general schedule.
     *
     * @param p_sAgendaID id of the agenda (format bktXXXXXXX)
     *
     * @return
     * on success, if xml was chosen:
     * - <agenda>
     *   - <status>true</status>
     * - </agenda>
     *
     * - on success, if json was chosen:
     *   - {"agenda":
     *     - {
     *       - "status":true,
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <agenda>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </agenda>
     *
     * - on failure, if json was chosen:
     *   - {"agenda":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */    
    public String setAgendaToCompanySchedule(String p_sAgendaID, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="setagendatocompanyschedule/"+_sPublicKey;
            String postParams="p_sAgendaID="+p_sAgendaID;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
    }
    
     /**
     * This function sets the cellphone of one of your company's users (if a user's ID is given) or the cellphone of your company (if no ID is given).
     *
     * Since cellphone numbers have to be always confirmed, the user or company will be set
     * as INACTIVE, and a new confirmation code will be returned by the function
     * and sent to the user's or company's cellphone.
     * Please store that code for later confirmation of the phone.
     * 
     * @see confirmphone
     *
     * @param p_sPhone new phone number
     * @param p_sUserID optional, id of your user (format bktXXXXXXXXX)
     *
     * @return
     * - on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <phone>
     *       - <status>true</status>
     *       - <validatephonekey>the new activation code</validatephonekey>
     *     - </phone>
     *
     * - on success, if json was chosen:
     *   - {"phone":
     *     - {
     *       - "status":true,
     *       - "validatephonekey":the new activation code,
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <phone>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </phone>
     *
     * - on failure, if json was chosen:
     *   - {"phone":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */    
    public String setPhoneAndSendValidate(String p_sPhone,String p_sUserID, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="setphoneandsendvalidate/"+_sPublicKey;
            String postParams="p_sPhone="+p_sPhone+"&p_sUserID="+p_sUserID;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
    }
    
    /**
     * This function updates data of an agenda of your company.
     * An agenda ID must be provided. It can be the ID of the agenda in Bookitit or, if you are synchronizing or integrating agendas between your software and Bookitit, the ID of the agenda in your software (we call it Synchro ID).
     * 
     * @param p_sAgendaID ID of the agenda (format bktXXXXXXX)
     * @param p_sSynchroID ID of the agenda in your software
     * @param p_sName agenda name (optional)
     * @param p_sPhone agenda phone (optional)
     * @param p_sEmail agenda email (optional)
     * @param p_sPublic (optional) indicates if the agenda is "public" or "private". Default is "public"
     * 
     * @return
     * on success, if xml was chosen:
     * - <agenda>
     *   - <status>true</status>
     * - </agenda>
     *
     * - on success, if json was chosen:
     *   - {"agenda":
     *     - {
     *       - "status":true,
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <agenda>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </agenda>
     *
     * - on failure, if json was chosen:
     *   - {"agenda":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     */    
    public String updateAgenda(String p_sAgendaID,String p_sSynchroID,String p_sName,String p_sPhone,String p_sEmail,String p_sPublic, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="updateagenda/"+_sPublicKey;
            String postParams="p_sAgendaID="+p_sAgendaID+"&p_sSynchroID="+p_sSynchroID+"&p_sName="+p_sName+"&p_sPhone="+p_sPhone+"&p_sEmail="+p_sEmail+"&p_sPublic="+p_sPublic;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
    }
    
     /**
     * This function updates the configuration for an agenda of your company.
     *
     * @param p_sAgendaID id of the agenda (format bktXXXXXXX)
     * @param p_iMinAdvanceMakeEvent integer, the minimum advance that users can make reservations with (in days)
     * @param p_iMinAdvanceSeeAgenda integer, the maximum anticipation that users can see the free hours with (in days)
     * @param p_iMinAdvanceCancelEvent integer, the minimum advance that users can cancel reservations with (in days)
     * @param p_sConfirmEvents indicates if the company has to manually confirm the reservations. Options are "yes" or "no", default is "no".
     * @param p_sUserValidate indicates if the users need to validate before making reservations. Options are "yes" or "no", default is "no".
     * @param p_sKeySendMethod indicates how to send the activation keys to users. Options are "sms" and "email", default is "email".
     * @param p_iWidgetIntervalSize how to set the interval between reservation hours in the widget. Options are:
     *  -  "service": Interval time depends on the duration of selected service
     *  -  any number: in minutes, Interval time is globally set
     *
     * @return
     * on success, if xml was chosen:
     * - <agenda>
     *   - <status>true</status>
     * - </agenda>
     *
     * - on success, if json was chosen:
     *   - {"agenda":
     *     - {
     *       - "status":true,
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <agenda>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </agenda>
     *
     * - on failure, if json was chosen:
     *   - {"agenda":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */    
    public String updateAgendaConfiguration(String p_sAgendaID,int p_iMinAdvanceMakeEvent,int p_iMinAdvanceSeeAgenda,int p_iMinAdvanceCancelEvent,String p_sConfirmEvents, String p_sUserValidate,String p_sKeySendMethod,int p_iWidgetIntervalSize, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="updateagendaconfiguration/"+_sPublicKey;
            String postParams="p_sAgendaID="+p_sAgendaID+"&p_iMinAdvanceMakeEvent="+p_iMinAdvanceMakeEvent+"&p_iMinAdvanceSeeAgenda="+p_iMinAdvanceSeeAgenda+
                    "&p_iMinAdvanceCancelEvent="+p_iMinAdvanceCancelEvent+"&p_sConfirmEvents="+p_sConfirmEvents+"&p_sUserValidate="+p_sUserValidate+
                    "&p_sKeySendMethod="+p_sKeySendMethod+"&p_iWidgetIntervalSize="+p_iWidgetIntervalSize;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
    }
    
     /**
     * This function updates an agenda-service connection of your company.
     * Agenda ID and service ID must be provided. They can be the IDs in Bookitit or, if you are synchronizing or integrating agendas between your software and Bookitit, the IDs of your software (we call it Synchro ID). If both are given, Synchro IDs have preference.
     * 
     * @param p_sAgendaID agenda ID (format bktXXXXXXX)
     * @param p_sServiceID service ID (format bktXXXXXXX)
     * @param p_sAgendaSynchroID agenda ID in your software (use in case of software integration)
     * @param p_sServiceSynchroID service ID in your software (use in case of software integration)
     * @param p_iDuration (optional) New duration of the service specific for this agenda, in minutes.
     * @param p_dPrice (optional) New price of the service specific for this agenda, in your currency
     *  
     * @return
     * on success, if xml was chosen:
     * - <agenda_service>
     *   - <status>true</status>
     * - </agenda_service>
     *
     * - on success, if json was chosen:
     *   - {"agenda_service":
     *     - {
     *       - "status":true,
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <agenda_service>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </agenda_service>
     *
     * - on failure, if json was chosen:
     *   - {"agenda_service":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     * 
     * 
     */    
    public String updateAgendaService(String p_sAgendaID,String p_sServiceID, String p_sAgendaSynchroID, String p_sServiceSynchroID, int p_iDuration,double p_dPrice, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="updateagendaservice/"+_sPublicKey;
            String postParams="p_sAgendaID="+p_sAgendaID+"&p_sServiceID="+p_sServiceID+
                    "&p_sAgendaSynchroID="+p_sAgendaSynchroID+"&p_sServiceSynchroID="+p_sServiceSynchroID+"&p_iDuration="+p_iDuration+"&p_dPrice="+p_dPrice;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
    }
    
     /**
     * This function is used to update data of a company in Bookitit.
     * Only administrator users, who have an administrator api key, can update
     * companies with this function.
     * Moreover, an administrator can only update the companies created by himself.
     *
     * @param p_sPublicKeyToModify Public key of the company to modify
     * @param p_sName Name of the company
     * @param p_sCellPhone Cellphone of the company
     * @param p_sPhone Phone of the company
     * @param p_sPassword Encrypted password (md5) of the company. If empty, the password won't be changed.
     * @param p_iIdCountry Country of the company
     * @param p_iIdRegion Region of the company
     * @param p_iIdCity City of the company
     * @param p_sAddress Address of the company
     * @param p_sPostalCode Postal code of the company
     * @param p_sWeb Url of the company
     *
     * @return
     *
     * - on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <company>
     *       - <status>true</status>
     *     - </company>
     *
     * - on success, if json was chosen:
     *   - {"company":
     *     - {
     *       - "status":true
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <company>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </company>
     *
     * - on failure, if json was chosen:
     *   - {"company":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     */    
    public String updateCompany(String p_sPublicKeyToModify,String p_sName,String p_sCellPhone,String p_sPhone,String p_sPassword,String p_sAddress,String p_sPostalCode,String p_sWeb, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="updatecompany/"+_sPublicKey;
            String postParams="p_sPublicKeyToModify="+p_sPublicKeyToModify+"&p_sName="+p_sName+"&p_sCellPhone="+p_sCellPhone+
                    "&p_sPhone="+p_sPhone+"&p_sPassword="+p_sPassword+"&p_sAddress="+p_sAddress+"&p_sPostalCode="+p_sPostalCode+"&p_sWeb="+p_sWeb;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
    }
    
     /**
     * This function updates the schedule of your company for a single weekday.
     * That is, at which time your company is open or closed.
     * Should be called once per day you want to update.
     * @Warning Only works for standard companies. 
     *
     * @param p_sWeekday the weekday you want to update (from "monday" to "sunday")
     * @param p_someOpeningTimesStrings an array of strings representing the opening times, grouped by periods of half an hour.
     *        For example, to indicate that the company is open from 10:00 to 12:00, and from 17:00 to 18:00, the array should be ["10:00", "10:30", "11:00", "11:30", "17:00", "17:30"]
     *
     * @return
     * - on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <schedule>
     *       - <status>true</status>
     *     - </schedule>
     *
     * - on success, if json was chosen:
     *   - {"schedule":
     *     - {
     *       - "status":true
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <schedule>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </schedule>
     *
     * - on failure, if json was chosen:
     *   - {"schedule":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */    
    public String updateCompanyScheduleDay(int p_sWeekDay, String[] p_someOpeningTimesStrings, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            
            String sUrl="updatecompanyscheduleday/"+_sPublicKey;
            
            if(p_sWeekDay < 1 || p_sWeekDay >7){
               p_sWeekDay=1; 
            }
            
            String postParams="var0="+p_sWeekDay;
        
            List<Integer> someOpeningTimesInts = new ArrayList<Integer>();

            for(String sOpeningTime : p_someOpeningTimesStrings){
                someOpeningTimesInts.add(this.renderMinutes(sOpeningTime));
            }

            int j=0;
            for(int i=0; i<=1410; i=i+30){
                postParams+="&var"+String.valueOf(j+1)+"="+i;
                if(someOpeningTimesInts.contains(i)){
                    postParams+="&var"+String.valueOf(j+2)+"=o";
                }
                else{
                    postParams+="&var"+String.valueOf(j+2)+"=c";
                }
                j=j+2;
            }

            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
    }
    
     /**
     * This function updates an event.
     *
     * @param p_sEventID id of the event (format bktXXXXXXXX)
     * @param p_sAgendaID id of the agenda (format bktXXXXXXXX)
     * @param p_sServiceID id of the service (format bktXXXXXXXX)
     * @param p_sEventSynchroID synchro ID of the event
     * @param p_sAgendaSynchroID synchro ID of the agenda
     * @param p_sServiceSynchroID synchro ID of the service
     * @param p_dStartDate start date of the event (YYYY-MM-DD)
     * @param p_dEndDate end date of the event (YYYY-MM-DD)
     * @param p_iStartTime start time of the event (in minutes, since 00:00)
     * @param p_iEndTime end time of the event (in minutes, since 00:00)
     * @param p_sTitle title of the event
     * @param p_sDescription description of the event
     *
     * @return
     * on success, if xml was chosen:
     * - <event>
     *   - <status>true</status>
     * - </event>
     *
     * - on success, if json was chosen:
     *   - {"event":
     *     - {
     *       - "status":true,
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <event>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </event>
     *
     * - on failure, if json was chosen:
     *   - {"event":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */    
    public String updateEvent(String p_sEventID,String p_sAgendaID,String p_sServiceID,String p_dStartDate,
            String p_dEndDate,int p_iStartTime,int p_iEndTime,String p_sTitle,String p_sDescription,
            String p_sEventSynchroID,String p_sAgendaSynchroID,String p_sServiceSynchroID, String p_sMode, boolean p_bSecure) 
            throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="updateevent/"+_sPublicKey;
            String postParams="p_sEventID="+p_sEventID+"&p_sAgendaID="+p_sAgendaID+"&p_sServiceID="+p_sServiceID+"&p_dStartDate="+p_dStartDate+
                    "&p_dEndDate="+p_dEndDate+"&p_iStartTime="+p_iStartTime+"&p_iEndTime="+p_iEndTime+"&p_sTitle="+p_sTitle+"&p_sDescription="+p_sDescription+
                    "&p_sEventSynchroID="+p_sEventSynchroID+"&p_sAgendaSynchroID="+p_sAgendaSynchroID+"&p_sServiceSynchroID="+p_sServiceSynchroID;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
    }
    
    /**
     * This function updates the status of an event. 
     *
     * @param p_sEventID id of the event (format bktXXXXXXX)
     * @param p_sStatus the new status of the event, options are "canceled", "completed", "pending", "pending_accepted", "pending_rejected", "in_payment", "wiretransfer" and "time_block"
     *
     * @return
     * on success, if xml was chosen:
     * - <event>
     *   - <status>true</status>     
     * - </event>
     *
     * - on success, if json was chosen:
     *   - {"event":
     *     - {
     *       - "status":true     
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <event>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </event>
     *
     * - on failure, if json was chosen:
     *   - {"event":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */    
    public String updateEventStatus(String p_sEventID,String p_sStatus, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="updateeventstatus/"+_sPublicKey;
            String postParams="p_sEventID="+p_sEventID+"&p_sStatus="+p_sStatus;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
    }
    
     /**
     * This function updates an previously created order of a prepay service.
     *
     * @param p_sPublicKey Your public key
     * @param p_sOrderID id of the order (format bktXXXXXXX)
     * @param p_sServiceID id of the related service (format bktXXXXXXX)
     * @param p_sEventID id of the related event (format bktXXXXXXX)
     * @param p_sStatus  new status of the order. Options are:
     *  -  "not_paid": the initial status (default)
     *  -  "bank_selected": User has selected the bank.
     *  -  "itemized": The bank request the order with lines.
     *  -  "transaction_ok": The bank confirms the payment.
     *  -  "paid": The service is definitely paid.
     *  -  "error": There has been a problem with the payment.
     *  -  "wiretransfer": This order is paid by wire transfer.
     * @param p_sBank name of the bank. Current options are:
     *  -  "paypal" (default)
     *  -  "pasat4b"
     * @param p_sStore id of the store to which the payment is made (more concrete than the bank)
     *
     * @return
     * on success, if xml was chosen:
     * - <order>
     *   - <status>true</status>
     *   - <id>id of the created agenda (format bktXXXXXXXX)</id>
     * - </order>
     *
     * - on success, if json was chosen:
     *   - {"order":
     *     - {
     *       - "status":true,
     *       - "id":id of the created service (format bktXXXXXXXX)
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <order>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </order>
     *
     * - on failure, if json was chosen:
     *   - {"order":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */    
    public String updateOrder(String p_sOrderID, String p_sServiceID, String p_iEventID, String p_sStatus, String p_sBank, String p_sStore, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="updateorder/"+_sPublicKey;
            String postParams="p_sOrderID="+p_sOrderID+"&p_sServiceID="+p_sServiceID+"&p_iEventID="+p_iEventID+"&p_sStatus="+p_sStatus+"&p_sBank="+p_sBank+"&p_sStore="+p_sStore;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
    }
    
     /**
     * This function updates data of a service of your company.
     * A service ID must be provided. It can be the ID of the service in Bookitit or, if you are synchronizing or integrating services between your software and Bookitit, the ID of the service in your software (we call it Synchro ID).
     * 
     * @param p_sPublicKey Your public key
     * @param p_sServiceID ID of the service (format bktXXXXXXX)
     * @param p_sSynchroID ID of the service in your local software
     * @param p_sName service name (optional)
     * @param p_sDuration service duration in minutes (optional)
     * @param p_sPrice service price (format XX.XX)(optional)
     * @param p_sPublic (optional) indicates if the service is "public" or "private". Default is "public"
     * @param p_sPrepay (optional) indicates is the services is "prepay" or "not_prepay". Default is "not_prepay"
     * @param p_sCurrency (optional) in which currency is the price, accepted values are "EUR" for euro "GBP" for british pound and "USD" for United States dollar. Default is euro.
     * 
     * @return
     * on success, if xml was chosen:
     * - <order>
     *   - <status>true</status>
     * - </order>
     *
     * - on success, if json was chosen:
     *   - {"order":
     *     - {
     *       - "status":true,
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <order>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </order>
     *
     * - on failure, if json was chosen:
     *   - {"order":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     * 
     */    
    public String updateService(String p_sServiceID,String p_sSynchroID,String p_sName,String p_sDuration,String p_sPrice,String p_sPublic,String p_sPrepay,String p_sCurrency, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="updateservice/"+_sPublicKey;
            String postParams="p_sServiceID="+p_sServiceID+"&p_sSynchroID="+p_sSynchroID+
                    "&p_sName="+p_sName+"&p_sDuration="+p_sDuration+"&p_sPrice="+p_sPrice+"&p_sPublic="+p_sPublic+"&p_sPrepay="+p_sPrepay+"&p_sCurrency="+p_sCurrency;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
    }
    
    /**
    * This function updates a social user given his basic data.
    *
    * @param p_sUserID The id of the user you want to update (format bktXXXXXXX)
    * @param p_sName Name of the user
    * @param p_sPhone Phone number of the user
    * @param p_sEmail email of the user
    *
    * @return
    * - on success, if xml was chosen:
    *   - <?xml version='1.0' encoding='utf-8'?>
    *     - <user>
    *       - <status>true</status>
    *     - </user>
    *
    * - on success, if json was chosen:
    *   - {"user":
    *     - {
    *       - "status":true,
    *     - }
    *   - }
    *
    * - on failure, if xml was chosen:
    *   - <?xml version='1.0' encoding='utf-8'?>
    *     - <user>
    *       - <status>false</status>
    *       - <id>error id</id>
    *       - <message>error message</message>
    *     - </user>
    *
    *
    * - on failure, if json was chosen:
    *   - {"user":
    *     - {
    *       - "status":false,
    *       - "id":error id,
    *       - "message":error message
    *     - }
    *   - }
    *
    * @see addsocialuser
    * @see setphoneandsendvalidate
    */    
    public String updateSocialUser(String p_sUserID,String p_sName,String p_sPhone,String p_sEmail, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="updatesocialuser/"+_sPublicKey;
            String postParams="p_sUserID="+p_sUserID+"&p_sName="+p_sName+"&p_sPhone="+p_sPhone+"&p_sEmail="+p_sEmail;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
    }
    
    /**
    * This function updates a user given his basic data.
    * 
    * @param p_sUserID The id of the user you want to update (format bktXXXXXXX)
    * @param p_sName (optional) Name of the user. If you don't want to send this parameter set is as "".
    * @param p_sPhone (optional) Phone number of the user. If you don't want to send this parameter set is as "".
    * @param p_sEmail (optional) email of the user. If you don't want to send this parameter set is as "".
    * @param p_sPhone (optional) Phone number of the user. If you don't want to send this parameter set is as "".
    * @param p_sDocument (optional) Document number of the user. If you don't want to send this parameter set is as "".
    * @param p_sAddress (optional) User address. If you don't want to send this parameter set is as "".
    * @param p_someCustomFields (optional) If your company account is using custom fields for the users, you can set it there. Remember that you have to maintain the index label.
            HashMap<String, String> someHash = new HashMap<String, String>();            
            someHash.put("customvalidate1", "yourvalue");
            someHash.put("customvalidate2", "yourvalue");
            someHash.put("custom1", "yourvalue");
            someHash.put("custom2", "yourvalue");
            someHash.put("custom3", "yourvalue");
            someHash.put("custom4", "yourvalue");
            someHash.put("custom5", "yourvalue");    
    *
    *
    * @return
    * - on success, if xml was chosen:
    *   - <?xml version='1.0' encoding='utf-8'?>
    *     - <user>
    *       - <status>true</status>
    *     - </user>
    *
    * - on success, if json was chosen:
    *   - {"user":
    *     - {
    *       - "status":true,
    *     - }
    *   - }
    *
    * - on failure, if xml was chosen:
    *   - <?xml version='1.0' encoding='utf-8'?>
    *     - <user>
    *       - <status>false</status>
    *       - <id>error id</id>
    *       - <message>error message</message>
    *     - </user>
    *
    *
    * - on failure, if json was chosen:
    *   - {"user":
    *     - {
    *       - "status":false,
    *       - "id":error id,
    *       - "message":error message
    *     - }
    *   - }
    *
    * @see addsocialuser
    * @see setphoneandsendvalidate
    */    
    public String updateUser(String p_sUserID,String p_sMode, boolean p_bSecure, String p_sName,String p_sPhone,
            String p_sEmail, String p_sDocument, String p_sAddress, HashMap<String, String> p_someCustomFields) 
            throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="updateuser/"+_sPublicKey;
            String postParams="p_sUserID="+p_sUserID+"&p_sName="+p_sName+"&p_sPhone="+p_sPhone+"&p_sEmail="+p_sEmail+
                    "&p_sDocument="+p_sDocument+"&p_sAddress="+p_sAddress;
            if (p_someCustomFields != null) {
                Iterator<String> keySetIterator = p_someCustomFields.keySet().iterator();

                while(keySetIterator.hasNext()){
                  String key = keySetIterator.next();                  
                  postParams += "&p_someCustomFields["+key+"]="+p_someCustomFields.get(key);
                }
            }
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
    }
    
     /**
     * This function validates (checks) if user's (or company's) credentials are correct.
     * Company Phone API keys are returned (they are generated if they don't exist).
     * 
     * @param p_sUserMail User's email
     * @param p_sPassword User's enrypted (md5) password
     * @param p_sType The type of user, can be "administrator", "client", "company, "social". Default is "client";
     *
     * @return
     * on success, if xml was chosen:
     * - <validate>
     *   - <status>true</status>
     *   - <public_key>mobile public key</public_key>
     *   - <private_key>mobile private key</private_key>
     * - </validate>
     *
     * - on success, if json was chosen:
     *   - {"validate":
     *     - {
     *       - "status":true
     *       - "public_key":mobile public key
     *       - "private_key":mobile private key
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <validate>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </validate>
     *
     * - on failure, if json was chosen:
     *   - {"validate":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */    
    public String userValidate(String p_sUserMail, String p_sPassword, String p_sType, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="uservalidate/"+_sPublicKey;
            String postParams="p_sUserMail="+p_sUserMail+"&p_sPassword="+p_sPassword+"&p_sType="+p_sType;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
    }
    
     /**
     * This function checks if the amount of an order is valid
     * 
     * @param p_sOrderID id of the order (format bktXXXXXXX)
     * @param p_iAmount id of the related service (format bktXXXXXXX)
     *
     * @return
     * on success, if xml was chosen:
     * - <order>
     *   - <status>true</status>
     * - </order>
     *
     * - on success, if json was chosen:
     *   - {"order":
     *     - {
     *       - "status":true,
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <order>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </order>
     *
     * - on failure, if json was chosen:
     *   - {"order":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */    
    public String validateOrder(String p_sOrderID, double p_dAmount, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException{
            String sUrl="validateorder/"+_sPublicKey;
            String postParams="p_sOrderID="+p_sOrderID+"&p_iAmount="+p_dAmount;
            String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
            return sResult;            
    }
    
    
     /**
     * This function validates a professional user against bookitit through API. 
     * It could be used if we want to access from another web application or dashboard backoffice to your bookitit account without user and password. 
     * The function will return a dynamic url that will allow the access temporally. When you get this url, you will have to redirect the user from your server side to this url.
     * 
     * @param p_sUserEmail the email of the user account to access
     * @param p_sAccountType possible values "single" or "multicenter". It will log into a single account or multicenter. In this version only allowed single.
     * @param p_sMode xml or json
     * @param p_bSecure true for https, false for http
     * 
     * @return String dynamic url that will allow the access temporally.
     
     * on success, if xml was chosen:
     * - <auth>
     *   - <status>true</status>
     *   - <authenticated_url>the url</authenticated_url>
     * - </auth>
     *
     * - on success, if json was chosen:
     *   - {"auth":
     *     - {
     *       - "status":true
     *       - "authenticated_url": the url where the client software will redirect     
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <auth>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </auth>
     *
     * - on failure, if json was chosen:
     *   - {"auth":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */
    public String auth(String p_sUserEmail, String p_sAccountType, String p_sMode, boolean p_bSecure) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException  {
        String sUrl = "auth/"+_sPublicKey+"/"+p_sAccountType;        
        String postParams = "p_sUserEmail="+p_sUserEmail;
        String sResult = startConnection(sUrl, p_sMode, p_bSecure, "POST", postParams);
        return sResult;
    }
    
    
    
    /**
     * This function gets an event by id, locator or both
     *
     *
     * @param p_sMode xml or json
     * @param p_bSecure true for https, false for http
     * @param p_iEventId the event id (optional). You can use this parameter and locator, only event id or only locator.
     * @param p_sLocator the event locator (optional). 
     *
     * @return
     * -on success, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <event>
     *         - <status>true</status>     
     *         - <id>id of the event (format "bktXXXXXXX")</id>
     *         - <title>title of the event service, if exists. empty otherwise</title>
     *         - <description>description of the event, if exists. empty otherwise</description>
     *         - <comments>comments of the event, if exists. empty otherwise</comments>
     *         - <synchro_id>id of the event if it is shared with other software, if exists. empty otherwise</synchro_id>
     *         - <startdate>start date of the event (format YYYY-MM-DD)</startdate>
     *         - <enddate>end date of the event</enddate>    
     *         - <starttime>start date of the event (format HH:MM)</starttime>
     *         - <endtime>end date of the event</endtime>     
     *         - <updated>the date when the event was updated, if exists. empty otherwise</updated>
     *         - <coupon>the event has associated a coupon, if exists. empty otherwise</coupon>
     *         - <agenda_id>the agenda where the event was created (format "bktXXXXXXX")</agenda_id>
     *         - <service_id>the service linked to the event, if exists. empty otherwise (format "bktXXXXXXX")</service_id>
     *         - <people> the number of people that will go to the event, if exists. empty otherwise</people>
     *         - <locator>the locator of the event/appointment, if exists. empty otherwise. Each event has one locator</locator>     
     *         - <customevent1>event custom field 1, if exists. empty otherwise</customevent1>
     *         - <customevent2>event custom field 2, if exists. empty otherwise</customevent2>
     *         - <customeventX>event custom field X, if exists. empty otherwise</customeventX>
     *     - </event>
     *
     * - on success, if json was chosen:
     *   - {"event":
     *     - {
     *         - "status":false,     
     *         - id: id of the event (format "bktXXXXXXX")
     *         - title: title of the event service, if exists. empty otherwise
     *         - description: description of the event, if exists. empty otherwise
     *         - comments: comments of the event, if exists. empty otherwise
     *         - synchro_id: id of the event if it is shared with other software, if exists. empty otherwise
     *         - startdate: start date of the event (format YYYY-MM-DD)
     *         - enddate: end date of the event
     *         - starttime: start date of the event (format HH:MM)
     *         - endtime: end date of the event
     *         - updated: the date when the event was updated, if exists. empty otherwise
     *         - coupon: the event has associated a coupon, if exists. empty otherwise
     *         - agenda_id: the agenda where the event was created (format "bktXXXXXXX")
     *         - service_id: the service linked to the event, if exists. empty otherwise (format "bktXXXXXXX")
     *         - people: the number of people that will go to the event, if exists. empty otherwise
     *         - locator: the locator of the event/appointment, if exists. empty otherwise. Each event has one locator
     *         - customevent1: event custom field 1, if exists. empty otherwise
     *         - customevent2: event custom field 2, if exists. empty otherwise
     *         - customeventX: event custom field X, if exists. empty otherwise
     *     - }
     *   - }
     *
     * - on failure, if xml was chosen:
     *   - <?xml version='1.0' encoding='utf-8'?>
     *     - <event>
     *       - <status>false</status>
     *       - <id>error id</id>
     *       - <message>error message</message>
     *     - </event>
     *
     * - on failure, if json was chosen:
     *   - {"event":
     *     - {
     *       - "status":false,
     *       - "id":error id,
     *       - "message":error message
     *     - }
     *   - }
     *
     */
    public String getEvent(String p_sMode, boolean p_bSecure,
            String p_iEventId, String p_sLocator) throws java.io.IOException, NoSuchAlgorithmException, SignatureException, InvalidKeyException, URISyntaxException {
        String sUrl = "geteventbyid/"+this._sPublicKey;
        
        if(p_iEventId != null){
            sUrl += "/"+p_iEventId;
        }
        
        if(p_sLocator != null && !p_sLocator.isEmpty()){
            sUrl += "/"+p_sLocator;
        }
        
        String sResult = startConnection(sUrl, p_sMode, p_bSecure, "GET", null);
        return sResult;
    }

}
